/*  Perspective Prototype, x37v.com 2014
 * 
 *  This is a prototype example of simulating minor perspective changes, designed for realtime interaction.
 *  Note that the use of the depth map to generate the output points causes empty pixels in the output array, 
 *  and consequently creates a 'topographical map' style effect.
 * 
 *  Code by Alex Mesker, x37v.com
 *  Depth map image by Sarah Keith, cetenbaath.com
 *
 * 'Face of the Man' image by George Hodan
 *  from: http://www.publicdomainpictures.net/view-image.php?image=19842&picture=face-of-the-man
 */

PImage input, depth, output;

float offsetX = 0;
float offsetY = 0;
float xDisp, yDisp, tempBright, brightMap;

int newPosY, newPosX, newPos, arrayLength, currPos;
int magnitude = 32; // the amount to scale the pixels from the depth map.  32 is natural, 100 is beautifully horrid

color srcColor, brightColor;
boolean auto = false;
 
void setup() {
  size(640, 360); // my source is 1280 x 720, but running at that res produces low fps.  960x540 runs at > 30fps on my laptop
  background(0);
  //noLoop();
  frameRate(60);
  input = loadImage("georgehodan-large.jpg");
  depth = loadImage("depthmap-detail.jpg");
  
  input.resize(width, height);
  depth.resize(width, height);
  output = new PImage(width, height);

  input.loadPixels();
  depth.loadPixels();
  
  arrayLength = width * height;
}

void draw() {
  //background(0);
  frame.setTitle(int(frameRate) + " fps");  // show framerate in title bar
  output = new PImage(width, height); // clear the output image buffer - commenting this line out is a hacky way to circumvent the 'topographical map' effect
  
  if(auto) {
    yDisp = magnitude*cos(radians(offsetY));  // make the face rotate with a displacement magnitude
    xDisp = magnitude*sin(radians(offsetX));
    offsetX = (offsetX + 1) % 360;
    offsetY = (offsetY + 1) % 360;
  } else {
    yDisp = map(mouseY, 0, height, -1 * magnitude * height/width, magnitude * height/width);
    xDisp = map(mouseX, 0, width, -1 * magnitude, magnitude);
  }

  for(int x = 0; x < width; x++) {
    for(int y = 0; y < height; y++) {  
      genPixels(x, y);
    } // end forloop
  } // end for loop

  output.updatePixels();
  //image(depth, 0, 0, width, height);  // show depthmap
  //image(input, 0, 0, width, height); // show source image
  //output.filter(BLUR, 1);
  image(output, 0, 0, width, height);
}

void genPixels(int x, int y) {
  currPos = y * width + x;
  srcColor = input.pixels[currPos]; // get the color of each pixel from the source
  brightColor = depth.pixels[currPos];
  brightMap = brightness(brightColor); // a brightness value [0. - 255.] from the 'depth' image
  
  if(int(brightMap) == 0) {
    output.pixels[currPos] = srcColor; // if there is no brightness in the depthmap, there is no need to adjust placement of pixels
  } else {
    brightMap /= 255; // normalise brightness [0. - 1.]
    newPosY = round(y + brightMap*yDisp);
    newPosX = round(x + brightMap*xDisp);
    newPos = newPosY * width + newPosX;
    if((newPos >= 0) && (newPos < arrayLength)) {
      output.pixels[newPos] = srcColor;       
    }
  }
}

void mouseMoved() {
  if(!auto) {
    offsetX = mouseX;
    offsetY = mouseY;
  }
}

void mouseClicked() {
   auto = !auto;
}
